#! /usr/bin/python
# -*- coding: utf-8 -*-

import logging
logger = logging.getLogger(__name__)

import numpy as np
from . import dili


def get_nlabels(slab, labels, labels_meta=None, return_mode="num", return_first=False):
    """
    Get one or more labels, create a new one if necessary and return its numeric value.

    Look at the get_nlabel function for more details.

    :param slab:
    :param labels:
    :param labels_meta:
    :param return_mode: "num" or "str" or "both". Both means (numlabel, strlabel).
    :param return_first: Return just first found label
    :return:
    """

    if type(labels) not in (list, np.ndarray):
        labels = [labels]
        labels_meta = [labels_meta]
        return_first = True

    if labels_meta is None:
        labels_meta = [None] * len(labels)

    nlabels = []
    for label, label_meta in zip(labels, labels_meta):
        nlab = get_nlabel(slab, label, label_meta, return_mode=return_mode)
        nlabels.append(nlab)

    if return_first:
        nlabels = nlabels[0]
    return nlabels


def get_nlabel(slab, label, label_meta=None, return_mode="num"):
    """
    Add label if it is necessery and return its numeric value.

    If "new" keyword is used and no other information is provided, the max + 1 label is created.
    If "new" keyword is used and additional numeric info is provided, the number is used also as a key.
    :param return_mode: Set requested label return type. "int", "num", "numeric" or "str" or "both".
    "both" means (numlabel, strlabel).
    :param label: string, number or "new"
    :param label_meta: string, number or "new
    :return:
    """
    # todo add add_new and dont_add parameters to have fine control over adding new keys in slab
    numlabel = None
    strlabel = None
    if type(label) == str:
        if label_meta is None:
            if label not in slab.keys():
                free_numeric_label = np.max(list(slab.values())) + 1
                if label == "new":
                    label = str(free_numeric_label)
                slab[label] = free_numeric_label
                strlabel = label
                numlabel = slab[label]
            else:
                strlabel = label
                numlabel = slab[label]
        else:
            if label == "new":
                label = str(label_meta)
            update_slab(slab, label_meta, label)
            strlabel = label
            numlabel = label_meta
    else:
        # it is numeric
        if label_meta is None:
            if label not in list(slab.values()):
                update_slab(slab, label, str(label))
                strlabel = str(label)
            else:
                strlabel = dili.dict_find_key(slab, label)

            numlabel = label

        else:
            if label_meta == "new":
                label_meta = str(label)
            update_slab(slab, label, label_meta)
            strlabel = label_meta
            numlabel = label
            # return label

    if return_mode in ("num", "int", "numeric"):
        return numlabel
    elif return_mode == "str":
        return strlabel
    elif return_mode == "both":
        return numlabel, strlabel
    else:
        logger.error("Unknown return_mode: " + str(return_mode))


def update_slab(slab, numeric_label, string_label):
    """ Add label to segmentation label dictionary if it is not there yet.

    :param numeric_label:
    :param string_label:
    :return:
    """

    slab_tmp = {string_label: numeric_label}
    slab.update(slab_tmp)
    # slab = slab_tmp
    logger.debug('self.slab')
    logger.debug(str(slab))


def add_slab_label_carefully(slab, numeric_label, string_label):
    """ Add label to slab if it is not there yet.

    :param numeric_label:
    :param string_label:
    :return:
    """
    DeprecationWarning("Function will be removed in the future. Use get_nlabel instead.")
    get_nlabel(slab, numeric_label, string_label)


def add_missing_labels(segmentation, slab):
    labels = np.unique(segmentation)
    get_nlabels(slab, labels)





#! /usr/bin/env python
# -*- coding: utf-8 -*-

import logging

logger = logging.getLogger(__name__)
# import funkcí z jiného adresáře
import os.path

import unittest

import numpy as np
import os

# import io3d
import io3d.datasets
import imma.segmentation_labels as imsl


class ImageManipulationTest(unittest.TestCase):
    interactivetTest = False

    def test_get_nlabels(self):
        datap = io3d.datasets.generate_abdominal()
        data3d = datap["data3d"]
        segmentation = datap["segmentation"]
        newlab = imsl.get_nlabels(datap["slab"], "new", return_mode="str")
        self.assertEqual(type(newlab), str)

    def test_get_nlabel_new(self):
        datap = io3d.datasets.generate_abdominal()
        data3d = datap["data3d"]
        segmentation = datap["segmentation"]
        newlab = imsl.get_nlabel(datap["slab"], "new", return_mode="str")
        self.assertEqual(type(newlab), str)

        newlab = imsl.get_nlabel(datap["slab"], "new", 37, return_mode="str")
        self.assertEqual(type(newlab), str, "new label with defined number value")
        self.assertEqual(newlab, "37")

        newlab = imsl.get_nlabel(datap["slab"], 38, "new", return_mode="str")
        self.assertEqual(type(newlab), str, "new label with defined number value")
        self.assertEqual(newlab, "38")

        newlab = imsl.get_nlabel(datap["slab"], "new", return_mode="num")
        self.assertTrue(np.issubdtype(type(newlab), np.integer))

    def test_add_missing_label(self):
        datap = io3d.datasets.generate_abdominal()
        data3d = datap["data3d"]
        segmentation = datap["segmentation"]
        slab = datap["slab"]
        slab = {"none": 0, "liver": 1}
        imsl.add_missing_labels(segmentation, slab)
        self.assertGreater(len(slab), 2)

    def test_simple_get_nlabel(self):
        slab = {"liver": 1, "porta": 2}
        val = imsl.get_nlabel(slab, 2)
        self.assertEqual(val, 2)
        self.assertEqual(len(slab), 2)

    def test_simple_string_get_nlabel(self):
        slab = {"liver": 1, "porta": 2}
        val = imsl.get_nlabel(slab, "porta")
        self.assertEqual(val, 2)
        self.assertEqual(len(slab), 2)

    def test_simple_new_numeric_get_nlabel(self):
        slab = {"liver": 1, "porta": 2}
        val = imsl.get_nlabel(slab, 7)
        self.assertNotEqual(val, 1)
        self.assertNotEqual(val, 2)
        self.assertEqual(val, 7)

    def test_simple_new_string_get_nlabel(self):
        slab = {"liver": 1, "porta": 2}
        val = imsl.get_nlabel(slab, "cava")
        self.assertNotEqual(val, 1)
        self.assertNotEqual(val, 2)

    def test_simple_add_carefully(self):
        slab = {"liver": 1, "porta": 2}
        imsl.add_slab_label_carefully(slab, 5, "cava")
        self.assertEqual(slab["cava"], 5)

    def test_simple_string_get_nlabel_return_string(self):
        slab = {"liver": 1, "porta": 2}
        val = imsl.get_nlabel(slab, "porta", return_mode="str")
        self.assertEqual(val, "porta")

    def test_simple_numeric_get_nlabel_return_string(self):
        slab = {"liver": 1, "porta": 2}
        val = imsl.get_nlabel(slab, 2, return_mode="str")
        self.assertEqual(val, "porta")

    def test_get_nlabels_single_label(self):
        slab = {"liver": 1, "kindey": 15, "none": 0}
        labels = 1
        val = imsl.get_nlabels(slab, labels)
        self.assertEqual(val, 1)

    def test_get_nlabels_multiple(self):
        slab = {"liver": 1, "porta": 2}
        val = imsl.get_nlabels(slab, [2, "porta", "new", 7], return_mode="str")
        self.assertEqual(val[0], "porta")
        self.assertEqual(val[1], "porta")
        self.assertEqual(val[2], "3")
        self.assertEqual(val[3], "7")

    def test_get_nlabels_single(self):
        slab = {"liver": 1, "porta": 2}

        val = imsl.get_nlabels(slab, "porta", return_mode="int")
        self.assertEqual(val, 2)

    def test_get_nlabels_single_both(self):
        slab = {"liver": 1, "porta": 2}

        val = imsl.get_nlabels(slab, "porta", return_mode="both")
        self.assertEqual(val[0], 2)
        self.assertEqual(val[1], "porta")


if __name__ == "__main__":
    unittest.main()



#! /usr/bin/env python
# -*- coding: utf-8 -*-

import logging

logger = logging.getLogger(__name__)
# import funkcí z jiného adresáře
import os.path

import unittest

import numpy as np
import os

# import io3d
import io3d.datasets
import imma.labeled as imlb
import imma.image as imim


class LabeledTest(unittest.TestCase):


    def test_crop_from_specific_data(self):
        datap = io3d.datasets.generate_abdominal()
        data3d = datap["data3d"]
        segmentation = datap["segmentation"]
        crinfo_auto1 = imlb.crinfo_from_specific_data(segmentation, [5])
        crinfo_auto2 = imlb.crinfo_from_specific_data(segmentation, 5)
        crinfo_auto3 = imlb.crinfo_from_specific_data(segmentation, [5, 5, 5])

        crinfo_expected = [[0, 99], [20, 99], [45, 99]]

        self.assertEquals(crinfo_auto1, crinfo_expected)
        self.assertEquals(crinfo_auto1, crinfo_auto2)
        self.assertEquals(crinfo_auto1, crinfo_auto3)

    def test_crop_from_specific_data_with_slices(self):
        datap = io3d.datasets.generate_abdominal()
        data3d = datap["data3d"]
        segmentation = datap["segmentation"]
        crinfo_auto1 = imlb.crinfo_from_specific_data(segmentation, [5], with_slices=True)
        self.assertEqual(type(data3d[crinfo_auto1]), np.ndarray, "We are able to use slices in data.")

        crinfo_auto1 = imim.fix_crinfo(crinfo_auto1, with_slices=False)
        crinfo_expected = [[0, 99], [20, 99], [45, 99]]

        self.assertEquals(crinfo_auto1, crinfo_expected)

    def test_select_objects_by_seeds(self):
        shape = [12, 15, 12]
        data = np.zeros(shape)
        value1 = 1
        value2 = 1
        data[:5, :7, :6] = value1
        data[-5:, :7, :6] = value2

        seeds = np.zeros(shape)
        seeds[9, 3:6, 3] = 1

        selected = imlb.select_objects_by_seeds(data, seeds)
        # import sed3
        # ed =sed3.sed3(selected, contour=data, seeds=seeds)
        # ed.show()
        unique = np.unique(selected)
        #
        self.assertEqual(selected.shape[0], shape[0])
        self.assertEqual(selected.shape[1], shape[1])
        self.assertEqual(selected.shape[2], shape[2])
        self.assertEqual(selected[1, 1, 1], 0)
        self.assertEqual(selected[-2, 1, 1], 1)
        self.assertEqual(len(unique), 2)
        self.assertGreater(np.count_nonzero(data), np.count_nonzero(selected))

    def test_squeeze_labels(self):
        seeds = np.zeros([50, 60, 70])
        seeds[20:30, 20:30, 20:30] = 30
        seeds[20:30, 50:70, 20:30] = 60
        seeds[20:30, 10:200, 40:60] = 61
        squeezed_seeds = imlb.squeeze_labels(seeds)
        self.assertEqual(np.max(squeezed_seeds), 3)

    def test_squeeze_labels_with_negative_and_collision(self):
        seeds = np.zeros([50, 60, 70])
        seeds[20:30, 20:30, 20:30] = -10
        seeds[20:30, 50:70, 20:30] = 1
        seeds[20:30, 10:200, 40:60] = 61
        squeezed_seeds = imlb.squeeze_labels(seeds)
        self.assertEqual(np.max(squeezed_seeds), 3)

    def test_dist_segmentation(self):
        seeds = np.zeros([8, 10])
        seeds[1:7, 8] = 1
        seeds[6, 5] = 2
        seeds[2, 0] = 3

        import matplotlib.pyplot as plt
        # plt.imshow(seeds, interpolation="nearest")
        # plt.show()

        segm = imlb.distance_segmentation(seeds)

        # plt.imshow(segm, interpolation="nearest")
        # plt.show()

        self.assertEqual(segm[2, 2], 3)
        self.assertEqual(segm[5, 5], 2)

        # dist, inds = scipy.in

    def test_select_labels(self):
        datap = io3d.datasets.generate_abdominal()
        data3d = datap["data3d"]
        segmentation = datap["segmentation"]
        selection = imlb.select_labels(segmentation, 1)
        self.assertGreater(np.sum(selection), 50, "select at least few pixels")
        # crinfo1 = ima.crinfo_from_specific_data(segmentation, [5])
        # crinfo2 = ima.extend_crinfo(crinfo1, data3d.shape, 3)
        # self.assertEqual(type(data3d[crinfo2]), np.ndarray, "We are able to use slices in data with extended crinfo.")

    def test_select_labels_with_slab(self):
        datap = io3d.datasets.generate_abdominal()
        data3d = datap["data3d"]
        segmentation = datap["segmentation"]
        selection = imlb.select_labels(segmentation, "liver", slab=datap["slab"])
        self.assertGreater(np.sum(selection), 50, "select at least few pixels")


    def test_biggest_object(self):
        datap = io3d.datasets.generate_abdominal()
        data3d = datap["data3d"]
        segmentation = datap["segmentation"]
        seg_biggest = imlb.get_one_biggest_object(segmentation)
        # newlab = ima.get_nlabels(datap["slab"], "new", return_mode="str")
        self.assertEqual(type(seg_biggest), np.ndarray)
        self.assertTrue(np.array_equal(seg_biggest.shape, segmentation.shape))
        self.assertTrue(np.array_equal(seg_biggest.shape, segmentation.shape))

    def test_biggest_object_label(self):
        datap = io3d.datasets.generate_abdominal()
        data3d = datap["data3d"]
        segmentation = datap["segmentation"]
        slab = datap["slab"]
        # import io3d.datasets
        # import imma.image_manipulation as ima
        datap = io3d.datasets.generate_abdominal()
        bigges_area_label = imlb.max_area_index(segmentation)
        self.assertEqual(bigges_area_label, slab["liver"])

        # test also the old version
        bigges_area_label2 = imlb.max_area_index2(segmentation, 30)
        self.assertEqual(bigges_area_label, bigges_area_label2)

    @unittest.skip("This test just checks the time requirements")
    def test_biggest_object_label_timeit(self):
        import timeit
        t0 = timeit.timeit(
            setup="""
import io3d.datasets
import imma.image_manipulation as ima
datap = io3d.datasets.generate_abdominal()
            """,
            stmt="ima.max_area_index(datap['segmentation'], 100)", number=3)
        t1 = timeit.timeit(
            setup="""
import io3d.datasets
import imma.image_manipulation as ima
datap = io3d.datasets.generate_abdominal()
            """,
            stmt="ima.max_area_index2(datap['segmentation'], 100)", number=3)
        # seg_biggest_i = ima.max_area_index(segmentation, 100)
        # seg_biggest_i = ima.max_area_index(segmentation, 100)
        # %timeit
        # newlab = ima.get_nlabels(datap["slab"], "new", return_mode="str")
        # self.assertEqual(type(seg_biggest), np.ndarray)
        # self.assertEqual(np.array_equal(seg_biggest.shape), segmentation.shape)
        # self.assertEqual(np.array_equal(seg_biggest.shape), segmentation.shape)
        self.assertGreater(t1, t0)
        # print(t0, t1)

    def test_unique_labels_by_seeds(self):
        labeled = np.zeros([10, 10])
        labeled[:5, :5] = 1
        labeled[5:10, :5] = 5
        labeled[:5, 5:10] = 6
        labeled[5:10, 5:10] = 13

        seeds = np.zeros([10, 10])
        seeds[2, 2] = 1
        seeds[8, 2] = 1
        seeds[:3, 8] = 2
        seeds[8, 8:] = 5

        expected_keys = [1, 2, 5]
        expected_values = [[1, 5], [6], [13]]
        unl = imlb.unique_labels_by_seeds(labeled, seeds)
        keys = list(unl.keys())
        values = list(unl.values())

        self.assertTrue(np.array_equal(keys, expected_keys))
        self.assertTrue(np.array_equal(values[0], expected_values[0]))
        self.assertTrue(np.array_equal(values[1], expected_values[1]))
        self.assertTrue(np.array_equal(values[2], expected_values[2]))


if __name__ == "__main__":
    unittest.main()



#! /usr/bin/env python
# -*- coding: utf-8 -*-
# vim:fenc=utf-8

import logging

logger = logging.getLogger(__name__)
import numpy as np

import unittest
from collections import OrderedDict
from imma import dili


class DictListTestCase(unittest.TestCase):
    def generate_dict_data(self):
        data = {
            'a': 1,
            'b': 2,
            'c': {
                'aa': 11,
                'bb': 22,
                'cc': {
                    'aaa': 111
                }
            }
        }
        return data

    def test_ditc_flatten(self):
        data = {
            'a': 1,
            'b': 2,
            'c': {
                'aa': 11,
                'bb': 22,
                'cc': {
                    'aaa': 111
                }
            }
        }
        dct = dili.flatten_dict(data)
        dct = dict(dct)
        self.assertIn("cccaaa", dct.keys())

    def test_dict_flatten_with_separator(self):
        data = {
            'a': 1,
            'b': 2,
            'c': {
                'aa': 11,
                'bb': 22,
                'cc': {
                    'aaa': 111
                }
            }
        }
        dct = dili.flatten_dict(data, separator=";")
        dct = dict(dct)
        self.assertIn("c;cc;aaa", dct.keys())

    def test_dict_recursive_update(self):
        data = self.generate_dict_data()
        data_updated = dili.recursive_update(data, {"c": {"aa": 33}})
        self.assertEqual(data_updated["c"]["aa"], 33)

    def test_ndarray_to_list(self):
        data = self.generate_dict_data()
        data["c"]["here is ndarray"] = np.asarray([1, 2, 5])

        data_with_list = dili.ndarray_to_list_in_structure(data)
        self.assertEqual(type(data_with_list["c"]["here is ndarray"]), list)

    def test_dict_split(self):
        data = self.generate_dict_data()
        ab, c = dili.split_dict(data, ["a", "b"])
        self.assertIn("a", ab.keys())
        self.assertIn("b", ab.keys())
        self.assertIn("c", c.keys())

    def test_dict_subdict(self):
        data = self.generate_dict_data()

        ab = dili.subdict(data, ["a", "b"])
        self.assertIn("a", ab.keys())
        self.assertIn("b", ab.keys())
        # self.assertIn("c", c.keys())

    def test_split_dict_ordered(self):
        data = OrderedDict([('pear', 1), ('orange', 2), ('banana', 3), ('apple', 4)])
        ab, c = dili.split_dict(data, ["pear", "banana"])
        self.assertIn("pear", ab.keys())
        self.assertIn("banana", ab.keys())
        self.assertIn("apple", c.keys())

    def test_kick_from_dict(self):
        data = self.generate_dict_data()
        dct = dili.kick_from_dict(data, ["a", "b"])
        self.assertNotIn("a", dct.keys())
        self.assertNotIn("b", dct.keys())
        self.assertIn("c", dct.keys())

    def test_find_subsring_in_list(self):
        lst = ["auto", "veloco", "toto", "cola"]
        output = dili.list_contains(lst, "co")

        self.assertIn("veloco", output)
        self.assertIn("cola", output)

    def test_list_filter(self):
        lst = ["aa", "sss", "aaron", "rew"]
        output = dili.list_filter(lst, notstartswith="aa")
        self.assertTrue(["sss", "rew"] == output)

    def test_dict_find_key(self):
        slab = {"liver": 1, "porta": 2}
        val = dili.dict_find_key(slab, 2)
        self.assertEqual(val, "porta")

    # def test_dict_find_key_error(self):
    #     slab={"liver": 1, "porta": 2}
    #     val = dili.dict_find_key(slab, 3)
    #     self.assertEqual(val, "porta")
    def sort_list_data(self):
        dct = [
            {"name": "mira", "age": 34, "height": 172.0, "weight": 75},
            {"name": "kamca", "age": 25, "height": 152.0, "weight": 55},
            {"name": "bob", "age": 34, "height": 183.0, "weight": 85},
            {"name": "pavel", "age": 34, "height": 179.0, "weight": 98},
            {"name": "veru", "age": 25, "height": 162.0, "weight": 60},
            {"name": "pepa", "age": 39, "height": 182.0, "weight": 130},
        ]
        return dct

    def test_sort_list_of_dicts(self):
        dct = self.sort_list_data()

        dct = dili.sort_list_of_dicts(dct, keys=["age", "height"])
        self.assertEqual(dct[0]["name"], "kamca")
        self.assertEqual(dct[1]["name"], "veru")
        self.assertEqual(dct[2]["name"], "mira")
        self.assertEqual(dct[-1]["name"], "pepa")

    def test_sort_list_of_dicts_single_key(self):
        dct = self.sort_list_data()
        dct = dili.sort_list_of_dicts(dct, keys="height")
        self.assertEqual(dct[0]["name"], "kamca")
        # self.assertEqual(dct[1]["name"], "veru")
        # self.assertEqual(dct[2]["name"], "mira")
        self.assertEqual(dct[-1]["name"], "bob")

    def test_ordered_dict_to_dict(self):
        from collections import OrderedDict

        od = OrderedDict()
        od["klkj"] = 1
        od["here is dict"] = {'s': 1, 1: 17}
        od["list"] = ["uuu", 146, ["sdf", 18]]
        od2 = OrderedDict()
        od2["as"] = 1
        od2["as2"] = 1
        od["ordered dict"] = od2

        normal_dict = dili.ordered_dict_to_dict(od)
        self.assertEqual(type(normal_dict), dict)

    def test_drop_duplicates(self):
        import pandas as pd

        df = pd.DataFrame({"col1": [5, 5, 1], "col2": [2, 2, 3], "time1": [1, 7, 9], "time2": [9, 7, 5]})
        dfn = dili.df_drop_duplicates(df, "time")
        # self.assertEqual(len(dfn.keys()), 2)
        self.assertEqual(len(dfn), 2)


def main():
    unittest.main()



#! /usr/bin/env python
# -*- coding: utf-8 -*-

import logging

logger = logging.getLogger(__name__)
# import funkcí z jiného adresáře
import os.path

import unittest

import numpy as np
import os

# import io3d
import io3d.datasets
import imma.image_manipulation as ima


class ImageManipulationTest(unittest.TestCase):
    interactivetTest = False

    # interactivetTest = True


    def test_seeds_inds(self):
        datap = io3d.datasets.generate_abdominal()
        seeds = datap["seeds"]
        data3d = datap["data3d"]
        sind = ima.as_seeds_inds(seeds, data3d.shape)
        self.assertGreater(seeds[sind][0], 0, "First found seed should be non zero")


if __name__ == "__main__":
    unittest.main()
